<?php

App::uses('BuzzDiaryAccountsAppController', 'BuzzDiaryAccounts.Controller');
App::uses('CustomerAddress', 'BuzzCustomers.Model');

class CampBookingsController extends BuzzDiaryAccountsAppController {

/**
 * Components
 *
 * @var array
 */
	public $components = array(
		'Transactions.Transactions'
	);

/**
 * Book and pay for a camp.
 *
 * @param int $campId ID of camp being booked
 * @return void
 */
	public function book($campId) {
		$CampBooking = $this->{$this->modelClass};

		$data = $CampBooking->Camp->getBooking($campId);
		if (empty($data)) {
			throw new NotFoundException();
		} elseif ($data['Camp']['remaining_availability'] === 0) {
			$this->Session->setFlash(__('Sorry, this camp is now fully booked'));
			return $this->redirect(['controller' => 'camps', 'action' => 'index']);
		}

		$this->set(compact('data'));

		if (!empty($this->request->data)) {

			if ($CampBooking->createBooking($campId, $this->request->data) === true) {
				// Take payment.
				$this->_pay();
			} else {
				$this->Session->setFlash(
					__d('buzz_camps', 'Please correct the errors below'),
					'flash_fail'
				);
			}

		} else {

			// Set the address field defaults (these are configured by the
			// CustomerAddress plugin to make it easier to override for each site).
			$this->request->data['AccountCredit']['CustomerAddress']['country_id'] = CustomerAddress::getDefaultCountry();
			$this->request->data['AccountCredit']['CustomerAddress']['us_state_id'] = CustomerAddress::getDefaultUsState();

		}

		$this->_populateLookups();
		$this->view = 'BuzzCamps./CampBookings/book';

		return;
	}

/**
 * Processes the payment and triggers the completion of the payment on success
 *
 * @return void
 */
	protected function _pay() {
		$CampBooking = $this->{$this->modelClass};

		// Get the latest basket.
		$id = $CampBooking->getBasketId();
		$basket = $CampBooking->findById($id);

		if ($basket['CampBooking']['total_paid'] > 0) {

			$transaction = $this->Transactions->takePayment(
				// Return URLs
				array(
					'return' => Router::url(['action' => 'redsys_callback'], true),
					'cancel' => Router::url(['action' => 'add_credit'], true)
				),
				// Calling record
				array(
					'model' => 'BuzzCamps.CampBooking',
					'model_id' => $basket['CampBooking']['id']
				),
				// Amount to be paid
				$basket['CampBooking']['total_paid'],
				// Items
				array(),
				// Extras
				array(
					'language' => Configure::read('Config.language')
				),
				0,
				Configure::read('BuzzPurchase.payment_gateway')
			);

			if (!empty($transaction)) {
				// Payment taken successfully, complete purchase.
				return $this->_completeBooking();
			} else {
				$this->Session->setFlash(
					__d('buzz_camps', 'There was a problem processing your payment, please check your details and try again'),
					'flash_fail'
				);
			}

		} else {
			// Nothing to pay, complete purchase.
			return $this->_completeBooking();
		}


		return;
	}

	public function redsys_callback() {
		$result = $this->Transactions->checkPayment();

		if ($result === true) {
			$this->loadModel('Transactions.Transaction');
			$this->Transaction->id = (int)$this->request->query['Ds_Order'];
			$basketId = $this->Transaction->field('model_id');
			return $this->_completeBooking([
				'vendorTxCode' => $this->request->query['Ds_Order'],
				'vpstxId' => $this->request->query['Ds_Signature'],
				'status' => 'OK',
				'txauthno' => $this->request->query['Ds_AuthorisationCode']
			]);
		} else {
			$this->Session->setFlash(
				__d('buzz_camps', 'There was a problem processing your payment, please try again'),
				'flash_fail'
			);
		}

		return;
	}

/**
 * Complete the credit transaction (after payment received)
 *
 * @return void Redirects to confirmation page
 */
	protected function _completeBooking($transaction = []) {
		$CampBooking = $this->{$this->modelClass};

		$basketId = $CampBooking->getBasketId();

		if (empty($basketId)) {
			throw new ForbiddenException();
		}

		$this->Session->write('CampBooking.hash', $CampBooking->hashBasketId());

		$CampBooking->completeBooking(true, $transaction);

		return $this->redirect([
			'action' => 'confirmation',
			$basketId
		]);
	}

/**
 * Credit confirmation page.
 *
 * @return void
 */
	public function confirmation($id) {
		$CampBooking = $this->{$this->modelClass};

		if ($CampBooking->hashBasketId($id) === $this->Session->read('CampBooking.hash')) {

			$this->Session->delete('CampBooking.hash');

			$booking = $CampBooking->readForView($id);

			if (Configure::check('BuzzCamps.camp_booking_confirmation_page_id') === true) {
				$this->assignPage(Configure::read('BuzzCamps.camp_booking_confirmation_page_id'));
			}

			$this->set(compact('booking'));
			$this->view = 'BuzzCamps.confirmation';

		} else {

			throw new ForbiddenException();

		}

		return;
	}

/**
 * Populate lookups.
 *
 * @return void
 */
	protected function _populateLookups() {
		$CampBooking = $this->{$this->modelClass};

		$countries = $CampBooking->CustomerAddress->Country->translatedList();
		$usStates = $CampBooking->CustomerAddress->UsState->find('list');

		$this->set(compact('countries', 'usStates'));
		return;
	}

}
